# This file was auto-generated by Fern from our API Definition.

import typing

ApiRole = typing.Union[
    typing.Literal["owner", "administrator", "billing_administrator", "corpus_administrator"], typing.Any
]
