# This file was auto-generated by Fern from our API Definition.

import typing
from .customer_specific_reranker import CustomerSpecificReranker
from .mmr_reranker import MmrReranker
from .none_reranker import NoneReranker

SearchReranker = typing.Union[CustomerSpecificReranker, MmrReranker, NoneReranker]
