# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class VectaraEnvironment:
    PRODUCTION: VectaraEnvironment

    def __init__(self, *, default: str, auth: str):
        self.default = default
        self.auth = auth


VectaraEnvironment.PRODUCTION = VectaraEnvironment(default="https://api.vectara.io", auth="https://auth.vectara.io")
