# This file was auto-generated by Fern from our API Definition.

from .search_parameters import SearchParameters
import typing
from .keyed_search_corpus import KeyedSearchCorpus
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchCorporaParameters(SearchParameters):
    """
    The parameters to search one or more corpora.
    """

    corpora: typing.Optional[typing.List[KeyedSearchCorpus]] = pydantic.Field(default=None)
    """
    The corpora that you want to search.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
