# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerSpecificReranker(UniversalBaseModel):
    """
    Reranker that is specific to the customer.
    """

    type: typing.Literal["customer_reranker"] = "customer_reranker"
    reranker_id: str = pydantic.Field()
    """
    The ID of the reranker. Current reranker that may be used by Scale customers is rnk_272725719.
    Do not specify the MMR reranker ID here, and instead use the MMR reranker object type.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
