# This file was auto-generated by Fern from our API Definition.

import typing
from .core_document import CoreDocument
from .structured_document import StructuredDocument

CreateDocumentRequest = typing.Union[CoreDocument, StructuredDocument]
