# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .llm import Llm
import pydantic
from .list_metadata import ListMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListLlMsResponse(UniversalBaseModel):
    llms: typing.Optional[typing.List[Llm]] = pydantic.Field(default=None)
    """
    List of LLMs.
    """

    metadata: typing.Optional[ListMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
